require "minitest"
require "minitest/autorun"
require "ioc"

class CarFixture
  attr_reader :engine
  def initialize(engine)
    @engine = engine
  end
end

class PropulsionFixture; end
class EngineFixture < PropulsionFixture; end
class MotorFixture < PropulsionFixture; end

describe IOC do
  it "can resolve a service class" do
    container = IOC::Container.new
    container.register(:myclass, EngineFixture)
    inst = container.resolve(:myclass)
    inst.must_be_instance_of(EngineFixture)
  end

  it "can resolve a service classes dependencies" do
    container = IOC::Container.new
    container.register(:engine, EngineFixture)
    container.register(:car, CarFixture)
    car = container.resolve(:car)
    car.engine.must_be_instance_of(EngineFixture)
  end

  it "caches resolved services" do
    container = IOC::Container.new
    container.register(:engine, EngineFixture)
    engine1 = container.resolve(:engine)
    engine2 = container.resolve(:engine)
    (engine1 === engine2).must_equal(true)
  end

  it "can resolve a service block" do
    myobject = "mystring"
    container = IOC::Container.new
    container.register(:myobject){|c| myobject }
    container.resolve(:myobject).must_equal(myobject)
  end

  it "complains when resolving unknown service" do
    container = IOC::Container.new
    proc { 
      container.resolve(:some_unknown_service) 
    }.must_raise(IOC::UnknownServiceError)
  end

  it "complains when service name registered more than once" do
    container = IOC::Container.new
    container.register(:myclass, EngineFixture)
    proc { 
      container.register(:myclass, EngineFixture) 
    }.must_raise(IOC::DuplicateServiceError)
  end

  describe "#validate!" do
    it "complains if dependencies are missing" do
      container = IOC::Container.new
      container.register(:car, CarFixture)
      proc {
        container.validate!
      }.must_raise(IOC::MissingDependenciesError)
    end
  end
end